@@ -1,15 +1,16 @@
 context("test-plot.R")
 
 test_that("Test plot.shapr", {
-
   if (requireNamespace("ggplot2", quietly = TRUE)) {
 
     # Example -----------
-    x <- matrix(c(4.98, 9.14, 4.03, 2.94, 5.33,
-                  6.575, 6.421, 7.185, 6.998, 7.147,
-                  4.0900, 4.9671, 4.9671, 6.0622, 6.0622,
-                  2.31, 7.07, 7.07, 2.18, 2.18),
-                ncol = 4
+    x <- matrix(c(
+      4.98, 9.14, 4.03, 2.94, 5.33,
+      6.575, 6.421, 7.185, 6.998, 7.147,
+      4.0900, 4.9671, 4.9671, 6.0622, 6.0622,
+      2.31, 7.07, 7.07, 2.18, 2.18
+    ),
+    ncol = 4
     )
 
     colnames(x) <- c("lstat", "rm", "dis", "indus")
